""" Defines an Image class and needs to be extended for fractal generation."""

class Image(object):
  """ Represents an Image, more specific a bitmap, of a given width and height. """


  def __init__( self, width, height ):
    """
    Constructs a new bitmap image of the given width and height.

    :ivar size: The width and height.
    :ivar data: The pixels initialized with zeros.
    """
    self.size = [ width, height ]
    self.data = []
    for i in xrange(width):
      self.data.append( [] )
      for j in xrange(height):
        self.data[i].append( 0 )


  def __getitem__(self, (i, j) ):
    """ Returns the pixel i,j. If i,j is out of bounds None is returned. """

    if( i >= 0 and i < self.size[0] and j >= 0 and j < self.size[1] ):
      return self.data[i][j]


  def __setitem__(self, (i, j), value ):
    """ Sets the pixel i,j to the given value. If i,j is out of bounds nothing is set. """

    if( i >= 0 and i < self.size[0] and j >= 0 and j < self.size[1] ):
      self.data[i][j] = value


  def __str__(self):
    """ Transforming this image into a string. First line has a header containing the class
    name and the dimensions, then for each row of pixels a new line is used.
    """

    result = "Image: " + str(self.size) + "\n"
    for i in xrange(0, self.size[0] - 1): # use xrange instead of range for memory efficiency
      for j in xrange( 0, self.size[1] - 1 ):
        result += str(self[i,j]).rjust(3) + " " # rjust to align numbers in cols
      result += "\n"
    return result

# # some tests
# img = Image( 14, 20 )
# img[3,3] = 42
# print img
