# -*- coding: utf-8 -*-
""" Renders a mandelbrot image """
from mandel1 import mandelfractal
import matplotlib.pyplot as plt # for image rendering
import numpy as np # for two dimensional bitmap array which is understood by matplotlib
from functools import partial # alter parameter list from a function to an alias
import multiprocessing # use multiprocessing tool

DEFAULT_WIDTH = 1024
DEFAULT_HEIGHT = 768
DEFAULT_DPI = 72

def computeRow(minX, minY, maxX, maxY, width, height, j):
    """ Computes a single row, given by parameter j of the image. """
    tmp = np.zeros( (width), dtype=np.uint8 )
    for i in xrange(width):
        x = float(i)/width
        y = float(j)/height
        c = complex(minX + x*(maxX-minX), minY + y*(maxY-minY))
        tmp[i] = mandelfractal(c)
    return tmp

def createImage(minX=-2.5, minY=-2.0, maxX=1.5, maxY=1.5, width=DEFAULT_WIDTH, height=DEFAULT_HEIGHT):
    """ Generates an Image for the given clip """
    # die partial function kann eine neue funktion von einer gegebenen ableiten, und nur
    # bestimme parameter verwenden, bzw. die anderen vorher festlegen (vgl. boost::bind)
    partialCalcRow = partial(computeRow, minX, minY, maxX, maxY, width, height)
    # Erzeuge einen Pool von n workers, wobei n == #cores
    pool=multiprocessing.Pool()
    # Für jedes j in xrange(height) wird der aufruf einem worker zugeteilt und das Ergebnis
    # anschließend an img angehangen
    img = pool.map(partialCalcRow, xrange(height))
    # wenn ein worker fertig ist, soll er den pool verlassen
    pool.close()
    # warte bis jeder den pool verlassen hat
    pool.join()
    return img

# # some tests
# plt.figure( figsize=(DEFAULT_WIDTH/DEFAULT_DPI,DEFAULT_HEIGHT/DEFAULT_DPI))
# img = createImage(-0.132, -0.992, -0.124, -0.983 )
# plt.imshow( img, cmap='flag' );plt.draw()
# plt.figure( figsize=(DEFAULT_WIDTH/DEFAULT_DPI,DEFAULT_HEIGHT/DEFAULT_DPI))
# plt.imshow( img, cmap='bone' );plt.draw()
# plt.figure( figsize=(DEFAULT_WIDTH/DEFAULT_DPI,DEFAULT_HEIGHT/DEFAULT_DPI))
# plt.imshow( img, cmap='PiYG' );plt.draw()
# plt.figure( figsize=(DEFAULT_WIDTH/DEFAULT_DPI,DEFAULT_HEIGHT/DEFAULT_DPI))
# plt.imshow( img, cmap='PuOr' );plt.draw()
# #plt.figure( figsize=(DEFAULT_WIDTH/DEFAULT_DPI,DEFAULT_HEIGHT/DEFAULT_DPI))
# #plt.imshow( createImage(-0.132, -0.992, -0.124, -0.983 ) )
# plt.show()
