# -*- coding: utf-8 -*-
""" This updates the bounds of the figure when the mouse button released within the window """
from mandel3b import createImage,plt
import matplotlib.cm # for getting a list of available colormaps
import random # for choosing randomly a colormap

# erzeuge array mit allen verfügbaren colormaps. Filtere solche herraus welche auf "_r" enden, da sie
# nur "reversed" sind. mit random.choice wird dann ein zufälliger String gewählt
COLORMAP=random.choice([m for m in plt.cm.datad if not m.endswith("_r")])
RES = 1000

def update(ax):
    """ Creates new image with updated dimensions. """
    # hole die aktuellen dimensionen des aktuellen aussschnitts. Das geht nur wenn extent richtig gesetzt wurde.
    xlim = ax.get_xlim()
    ylim = ax.get_ylim()
    # Erzeuge neues Bild für diesen Ausschnitt mit der Auflösung RES
    img = createImage(xlim[0], ylim[1], xlim[1], ylim[0], RES, RES )
    ax.clear()
    # lade bild in den zugehörigen Grafik Context und wende zufällige Colormap an
    ax.imshow(img, extent=[xlim[0], xlim[1], ylim[0], ylim[1]], zorder=0, cmap=COLORMAP)
    # zeichne bild
    ax.figure.canvas.draw()

# some tests
plt.imshow(createImage(-2.0, -1.0, 1.0, 1.0, RES, RES), extent=[-2.0, 1.0, -1.0, 1.0], cmap=COLORMAP) # start image
ax = plt.gca() # GetCurrentAxis of the plot
ax.figure.canvas.mpl_connect('button_release_event', lambda x: update(ax)) # whenever a button is released the scene gets updated
plt.show()
